<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform version="2.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:strip-space elements="*"/> 

<xsl:variable name="summary">
  <xsl:variable name="t-total" select="/trace/end/@time - /trace/@time"/>
  <stats t-total="{$t-total}">
  <xsl:variable name="data">
    <xsl:for-each-group select="//Instruction" group-by="concat(@file,@line,@element)">
                <xsl:sort select="@file"/>
                <xsl:sort select="@line"/>
                <xsl:sort select="@element"/>
            <Instruction>
            <xsl:copy-of select="@file"/>
            <xsl:copy-of select="@line"/>
            <xsl:copy-of select="@element"/>
            <xsl:variable name="count" select="count(current-group())"/>
            
            <xsl:variable name="group">
                <xsl:for-each select="current-group()">
                    <xsl:copy>
                    <xsl:copy-of select="@*"/>
                    <xsl:variable name="gross" select="end/@time - @time"/>
                    <xsl:attribute name="t">
                        <xsl:value-of select="$gross"/>
                    </xsl:attribute>
                    <xsl:attribute name="t2">
                        <xsl:value-of select="$gross * $gross"/>
                    </xsl:attribute>
                    <xsl:variable name="net" 
                                  select="$gross - sum(for $i in child::Instruction
                                                       return ($i/end/@time - $i/@time))"/>
                    <xsl:attribute name="net">
                        <xsl:value-of select="$net"/>
                    </xsl:attribute>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:variable name="t-sum" select="sum($group/Instruction/@t)"/>
            <xsl:variable name="t-min" select="min($group/Instruction/@t)"/>
            <xsl:variable name="t-max" select="max($group/Instruction/@t)"/>
            <xsl:variable name="t-avg" select="avg($group/Instruction/@t)"/>
            
            <xsl:variable name="t-M2" select="sum($group/Instruction/@t2) div $count"/>
            
            <xsl:variable name="log10" xmlns:m="java:java.lang.Math" select="m:log(10)"/>
            <xsl:attribute name="score">
                <xsl:value-of  xmlns:m="java:java.lang.Math"
                    select="m:log($t-sum * $count) div $log10"/>
            </xsl:attribute>
            <xsl:attribute name="count">
                <xsl:value-of select="$count"/>
            </xsl:attribute>
            <xsl:attribute name="t-sum">
                <xsl:value-of select="$t-sum"/>
            </xsl:attribute>
            <xsl:attribute name="t-min">
                <xsl:value-of select="$t-min"/>
            </xsl:attribute> 
            <xsl:attribute name="t-max">
                <xsl:value-of select="$t-max"/>
            </xsl:attribute> 
            <xsl:attribute name="t-avg">
                <xsl:value-of select="$t-avg"/>
            </xsl:attribute>
            <xsl:attribute name="t-std">
                <xsl:value-of xmlns:m="java:java.lang.Math" select="m:sqrt($t-M2 - ($t-avg * $t-avg))"/>
            </xsl:attribute>
            <xsl:attribute name="t-avg-net">
                <xsl:value-of select="sum($group/Instruction/@t) div $count"/>
            </xsl:attribute>
            </Instruction>
    </xsl:for-each-group>
  </xsl:variable>

  <xsl:for-each select="$data/Instruction">
    <xsl:sort select="@score" data-type="number" order="descending"/>
    <xsl:copy-of select="."/>
  </xsl:for-each>
  </stats>
</xsl:variable>

<xsl:template match="/">
  <xsl:apply-templates select="$summary" mode="report"/>
</xsl:template>

<xsl:template match="stats" mode="report">
<html>
  <head><title>Analysis of Stylesheet Execution Time</title></head>
  <body><h1>Analysis of Stylesheet Execution Time</h1>
  <p>Total time: <xsl:value-of select="@t-total"/> milliseconds</p>
  <h2>Time spent in each Instruction:</h2>
  <p>The table below is ordered based on a weighting function that considers both the
  frequency of execution and the average time spent in the instruction.</p>
  <table border="border" cellpadding="10">
    <thead>
        <tr>
          <th>file</th>
          <th>line</th>
          <th>instruction</th>
          <th>count</th>
          <th>average time</th>
          <th>total time</th>          
          </tr>
    </thead>
    <tbody>
      <xsl:for-each select="Instruction">
        <tr>
            <td><xsl:value-of select="@file"/></td>
            <td><xsl:value-of select="@line"/></td>
            <td><xsl:value-of select="@element"/></td>      
            <td align="right"><xsl:value-of select="@count"/></td>
            <td align="right"><xsl:value-of select="format-number(@t-avg, '#0.000')"/></td> 
            <td align="right"><xsl:value-of select="format-number(@t-sum, '#0.000')"/></td>              
        </tr>
      </xsl:for-each>
    </tbody>
  </table>
  </body>
  </html>
</xsl:template>          

</xsl:transform>
